@@ -19,6 +19,10 @@ from testfixtures.mock import parent_name, mock_call
 from unittest.mock import call as unittest_mock_call
 
 
+# Some common types that are immutable, for optimisation purposes within CompareContext
+IMMUTABLE_TYPEs = str, bytes, int, float, tuple, type(None)
+
+
 def diff(x: str, y: str, x_label: str = '', y_label: str = ''):
     """
     A shorthand function that uses :mod:`difflib` to return a
@@ -148,6 +152,8 @@ def compare_with_type(x, y, context: 'CompareContext') -> str:
     Return a textual description of the difference between two objects
     including information about their types.
     """
+    if type(x) is AlreadySeen and type(x.obj) is type(y) and x.obj == y:
+        return ''
     source = locals()
     to_render = {}
     for name in 'x', 'y':
@@ -486,7 +492,28 @@ def _shared_mro(x, y):
 _unsafe_iterables = str, bytes, dict
 
 
+class AlreadySeen:
+
+    def __init__(self, id_, obj, breadcrumb):
+        self.id = id_
+        self.obj = obj
+        self.breadcrumb = breadcrumb
+
+    def __repr__(self):
+        return f'<AlreadySeen for {self.obj!r} at {self.breadcrumb} with id {self.id}>'
+
+    def __eq__(self, other):
+        if isinstance(other, AlreadySeen):
+            return self.breadcrumb == other.breadcrumb
+        else:
+            return self.obj == other
+
+
 class CompareContext(object):
+    """
+    Stores the context of the current comparison in process during a call to
+    :func:`testfixtures.compare`.
+    """
 
     def __init__(
             self,
@@ -511,6 +538,7 @@ class CompareContext(object):
         self.options: Dict[str, Any] = options or {}
         self.message: str = ''
         self.breadcrumbs: List[str] = []
+        self._seen = {}
 
     def extract_args(self, args: tuple, x: Any, y: Any, expected: Any, actual: Any) -> List:
 
@@ -571,8 +599,24 @@ class CompareContext(object):
     def _separator(self) -> str:
         return '\n\nWhile comparing %s: ' % ''.join(self.breadcrumbs[1:])
 
+    def _break_loops(self, obj, breadcrumb):
+        # Don't bother with this process for simple, immutable types:
+        if isinstance(obj, IMMUTABLE_TYPEs):
+            return obj
+
+        id_ = id(obj)
+        breadcrumb_ = self._seen.get(id_)
+        if breadcrumb_ is not None:
+            return AlreadySeen(id_, obj, breadcrumb_)
+        else:
+            self._seen[id_] = breadcrumb
+            return obj
+
     def different(self, x: Any, y: Any, breadcrumb: str) -> Union[bool, Optional[str]]:
 
+        x = self._break_loops(x, breadcrumb)
+        y = self._break_loops(y, breadcrumb)
+
         recursed = bool(self.breadcrumbs)
         self.breadcrumbs.append(breadcrumb)
         existing_message = self.message
@@ -580,18 +624,18 @@ class CompareContext(object):
         current_message = ''
         try:
 
-            if not (self.strict or self.ignore_eq) and x == y:
-                return False
+            if type(y) is AlreadySeen or not (self.strict or self.ignore_eq):
+                try:
+                    if x == y:
+                        return False
+                except RecursionError:
+                    pass
 
             comparer: Comparer = self._lookup(x, y)
 
             result = comparer(x, y, self)
             specific_comparer = comparer is not compare_simple
 
-            if self.strict:
-                if x == y and not specific_comparer:
-                    return False
-
             if result:
 
                 if specific_comparer and recursed:
@@ -779,7 +823,7 @@ class Comparison(StatefulComparison):
             else:
                 attribute_dict.update(attributes)
         if isinstance(object_or_type, str):
-            container, method, name, c = resolve(object_or_type)
+            c = resolve(object_or_type).found
             if c is not_there:
                 raise AttributeError(
                     '%r could not be resolved' % object_or_type
