@@ -2,8 +2,11 @@ from datetime import date
 from datetime import datetime as d
 from datetime import timedelta
 from datetime import tzinfo
-from testfixtures import test_datetime, test_date
+from typing import cast, Type
+
+from testfixtures import mock_datetime, mock_date
 from testfixtures import replace, Replacer, compare, ShouldRaise
+from testfixtures.datetime import MockDateTime
 from testfixtures.tests import sample1
 from unittest import TestCase
 
@@ -32,56 +35,56 @@ class SampleTZInfo2(tzinfo):
 
 class TestDateTime(TestCase):
 
-    @replace('datetime.datetime', test_datetime())
+    @replace('datetime.datetime', mock_datetime())
     def test_now(self):
         from datetime import datetime
         compare(datetime.now(), d(2001, 1, 1, 0, 0, 0))
         compare(datetime.now(), d(2001, 1, 1, 0, 0, 10))
         compare(datetime.now(), d(2001, 1, 1, 0, 0, 30))
 
-    @replace('datetime.datetime', test_datetime())
+    @replace('datetime.datetime', mock_datetime())
     def test_now_with_tz_supplied(self):
         from datetime import datetime
         info = SampleTZInfo()
         compare(datetime.now(info), d(2001, 1, 1, 0, 4, tzinfo=SampleTZInfo()))
 
-    @replace('datetime.datetime', test_datetime(tzinfo=SampleTZInfo()))
+    @replace('datetime.datetime', mock_datetime(tzinfo=SampleTZInfo()))
     def test_now_with_tz_setup(self):
         from datetime import datetime
         compare(datetime.now(), d(2001, 1, 1))
 
-    @replace('datetime.datetime', test_datetime(tzinfo=SampleTZInfo()))
+    @replace('datetime.datetime', mock_datetime(tzinfo=SampleTZInfo()))
     def test_now_with_tz_setup_and_supplied(self):
         from datetime import datetime
         info = SampleTZInfo2()
         compare(datetime.now(info), d(2001, 1, 1, 0, 1, tzinfo=info))
 
-    @replace('datetime.datetime', test_datetime(tzinfo=SampleTZInfo()))
+    @replace('datetime.datetime', mock_datetime(tzinfo=SampleTZInfo()))
     def test_now_with_tz_setup_and_same_supplied(self):
         from datetime import datetime
         info = SampleTZInfo()
         compare(datetime.now(info), d(2001, 1, 1, tzinfo=info))
 
     def test_now_with_tz_instance(self):
-        dt = test_datetime(d(2001, 1, 1, tzinfo=SampleTZInfo()))
+        dt = mock_datetime(d(2001, 1, 1, tzinfo=SampleTZInfo()))
         compare(dt.now(), d(2001, 1, 1))
 
     def test_now_with_tz_instance_and_supplied(self):
-        dt = test_datetime(d(2001, 1, 1, tzinfo=SampleTZInfo()))
+        dt = mock_datetime(d(2001, 1, 1, tzinfo=SampleTZInfo()))
         info = SampleTZInfo2()
         compare(dt.now(info), d(2001, 1, 1, 0, 1, tzinfo=info))
 
     def test_now_with_tz_instance_and_same_supplied(self):
-        dt = test_datetime(d(2001, 1, 1, tzinfo=SampleTZInfo()))
+        dt = mock_datetime(d(2001, 1, 1, tzinfo=SampleTZInfo()))
         info = SampleTZInfo()
         compare(dt.now(info), d(2001, 1, 1, tzinfo=info))
 
-    @replace('datetime.datetime', test_datetime(2002, 1, 1, 1, 2, 3))
+    @replace('datetime.datetime', mock_datetime(2002, 1, 1, 1, 2, 3))
     def test_now_supplied(self):
         from datetime import datetime
         compare(datetime.now(), d(2002, 1, 1, 1, 2, 3))
 
-    @replace('datetime.datetime', test_datetime(None))
+    @replace('datetime.datetime', mock_datetime(None))
     def test_now_sequence(self, t):
         t.add(2002, 1, 1, 1, 0, 0)
         t.add(2002, 1, 1, 2, 0, 0)
@@ -91,7 +94,7 @@ class TestDateTime(TestCase):
         compare(datetime.now(), d(2002, 1, 1, 2, 0, 0))
         compare(datetime.now(), d(2002, 1, 1, 3, 0, 0))
 
-    @replace('datetime.datetime', test_datetime())
+    @replace('datetime.datetime', mock_datetime())
     def test_add_and_set(self, t):
         t.add(2002, 1, 1, 1, 0, 0)
         t.add(2002, 1, 1, 2, 0, 0)
@@ -101,8 +104,8 @@ class TestDateTime(TestCase):
         compare(datetime.now(), d(2002, 1, 1, 3, 0, 10))
         compare(datetime.now(), d(2002, 1, 1, 3, 0, 30))
 
-    @replace('datetime.datetime', test_datetime(None))
-    def test_add_datetime_supplied(self, t):
+    @replace('datetime.datetime', mock_datetime(None))
+    def test_add_datetime_supplied(self, t: Type[MockDateTime]):
         from datetime import datetime
         t.add(d(2002, 1, 1, 1))
         t.add(datetime(2002, 1, 1, 2))
@@ -118,11 +121,11 @@ class TestDateTime(TestCase):
 
     def test_instantiate_with_datetime(self):
         from datetime import datetime
-        t = test_datetime(datetime(2002, 1, 1, 1))
+        t = mock_datetime(datetime(2002, 1, 1, 1))
         compare(t.now(), d(2002, 1, 1, 1, 0, 0))
 
-    @replace('datetime.datetime', test_datetime(None))
-    def test_now_requested_longer_than_supplied(self, t):
+    @replace('datetime.datetime', mock_datetime(None))
+    def test_now_requested_longer_than_supplied(self, t: Type[MockDateTime]):
         t.add(2002, 1, 1, 1, 0, 0)
         t.add(2002, 1, 1, 2, 0, 0)
         from datetime import datetime
@@ -131,8 +134,8 @@ class TestDateTime(TestCase):
         compare(datetime.now(), d(2002, 1, 1, 2, 0, 10))
         compare(datetime.now(), d(2002, 1, 1, 2, 0, 30))
 
-    @replace('datetime.datetime', test_datetime(strict=True))
-    def test_call(self, t):
+    @replace('datetime.datetime', mock_datetime(strict=True))
+    def test_call(self, t: Type[MockDateTime]):
         compare(t(2002, 1, 2, 3, 4, 5), d(2002, 1, 2, 3, 4, 5))
         from datetime import datetime
         dt = datetime(2001, 1, 1, 1, 0, 0)
@@ -141,7 +144,7 @@ class TestDateTime(TestCase):
 
     def test_date_return_type(self):
         with Replacer() as r:
-            r.replace('datetime.datetime', test_datetime())
+            r.replace('datetime.datetime', mock_datetime())
             from datetime import datetime
             dt = datetime(2001, 1, 1, 1, 0, 0)
             d = dt.date()
@@ -150,9 +153,9 @@ class TestDateTime(TestCase):
 
     def test_date_return_type_picky(self):
         # type checking is a bitch :-/
-        date_type = test_date(strict=True)
+        date_type = mock_date(strict=True)
         with Replacer() as r:
-            r.replace('datetime.datetime', test_datetime(date_type=date_type,
+            r.replace('datetime.datetime', mock_datetime(date_type=date_type,
                                                          strict=True,
                                                          ))
             from datetime import datetime
@@ -166,7 +169,7 @@ class TestDateTime(TestCase):
     # do a manual try-finally with a replacer:
     def test_import_and_obtain_with_lists(self):
 
-        t = test_datetime(None)
+        t = mock_datetime(None)
         t.add(2002, 1, 1, 1, 0, 0)
         t.add(2002, 1, 1, 2, 0, 0)
 
@@ -179,36 +182,37 @@ class TestDateTime(TestCase):
         finally:
             r.restore()
 
-    @replace('datetime.datetime', test_datetime())
+    @replace('datetime.datetime', mock_datetime())
     def test_repr(self):
         from datetime import datetime
-        compare(repr(datetime), "<class 'testfixtures.tdatetime.tdatetime'>")
+        compare(repr(datetime), "<class 'testfixtures.datetime.MockDateTime'>")
 
-    @replace('datetime.datetime', test_datetime(delta=1))
+    @replace('datetime.datetime', mock_datetime(delta=1))
     def test_delta(self):
         from datetime import datetime
         compare(datetime.now(), d(2001, 1, 1, 0, 0, 0))
         compare(datetime.now(), d(2001, 1, 1, 0, 0, 1))
         compare(datetime.now(), d(2001, 1, 1, 0, 0, 2))
 
-    @replace('datetime.datetime', test_datetime(delta_type='minutes'))
+    @replace('datetime.datetime', mock_datetime(delta_type='minutes'))
     def test_delta_type(self):
         from datetime import datetime
         compare(datetime.now(), d(2001, 1, 1, 0, 0, 0))
         compare(datetime.now(), d(2001, 1, 1, 0, 10, 0))
         compare(datetime.now(), d(2001, 1, 1, 0, 30, 0))
 
-    @replace('datetime.datetime', test_datetime(None))
+    @replace('datetime.datetime', mock_datetime(None))
     def test_set(self):
         from datetime import datetime
+        datetime = cast(Type[MockDateTime], datetime)
         datetime.set(2001, 1, 1, 1, 0, 1)
         compare(datetime.now(), d(2001, 1, 1, 1, 0, 1))
         datetime.set(2002, 1, 1, 1, 0, 0)
         compare(datetime.now(), d(2002, 1, 1, 1, 0, 0))
         compare(datetime.now(), d(2002, 1, 1, 1, 0, 20))
 
-    @replace('datetime.datetime', test_datetime(None))
-    def test_set_datetime_supplied(self, t):
+    @replace('datetime.datetime', mock_datetime(None))
+    def test_set_datetime_supplied(self, t: Type[MockDateTime]):
         from datetime import datetime
         t.set(d(2002, 1, 1, 1))
         compare(datetime.now(), d(2002, 1, 1, 1, 0, 0))
@@ -222,60 +226,66 @@ class TestDateTime(TestCase):
             ))):
             t.set(d(2001, 1, 1, tzinfo=tzinfo))
 
-    @replace('datetime.datetime', test_datetime(None, tzinfo=SampleTZInfo()))
+    @replace('datetime.datetime', mock_datetime(None, tzinfo=SampleTZInfo()))
     def test_set_tz_setup(self):
         from datetime import datetime
+        datetime = cast(Type[MockDateTime], datetime)
         datetime.set(year=2002, month=1, day=1)
         compare(datetime.now(), d(2002, 1, 1))
 
-    @replace('datetime.datetime', test_datetime(None))
+    @replace('datetime.datetime', mock_datetime(None))
     def test_set_kw(self):
         from datetime import datetime
+        datetime = cast(Type[MockDateTime], datetime)
         datetime.set(year=2002, month=1, day=1)
         compare(datetime.now(), d(2002, 1, 1))
 
-    @replace('datetime.datetime', test_datetime(None))
+    @replace('datetime.datetime', mock_datetime(None))
     def test_set_tzinfo_kw(self):
         from datetime import datetime
-        with ShouldRaise(TypeError('Cannot add using tzinfo on tdatetime')):
+        datetime = cast(Type[MockDateTime], datetime)
+        with ShouldRaise(TypeError('Cannot add using tzinfo on MockDateTime')):
             datetime.set(year=2002, month=1, day=1, tzinfo=SampleTZInfo())
 
-    @replace('datetime.datetime', test_datetime(None))
+    @replace('datetime.datetime', mock_datetime(None))
     def test_set_tzinfo_args(self):
         from datetime import datetime
-        with ShouldRaise(TypeError('Cannot add using tzinfo on tdatetime')):
+        datetime = cast(Type[MockDateTime], datetime)
+        with ShouldRaise(TypeError('Cannot add using tzinfo on MockDateTime')):
             datetime.set(2002, 1, 2, 3, 4, 5, 6, SampleTZInfo())
 
-    @replace('datetime.datetime', test_datetime(None))
-    def test_add_kw(self, t):
+    @replace('datetime.datetime', mock_datetime(None))
+    def test_add_kw(self, t: Type[MockDateTime]):
         from datetime import datetime
         t.add(year=2002, day=1, month=1)
         compare(datetime.now(), d(2002, 1, 1))
 
-    @replace('datetime.datetime', test_datetime(None))
-    def test_add_tzinfo_kw(self, t):
+    @replace('datetime.datetime', mock_datetime(None))
+    def test_add_tzinfo_kw(self, t: Type[MockDateTime]):
         from datetime import datetime
-        with ShouldRaise(TypeError('Cannot add using tzinfo on tdatetime')):
+        datetime = cast(Type[MockDateTime], datetime)
+        with ShouldRaise(TypeError('Cannot add using tzinfo on MockDateTime')):
             datetime.add(year=2002, month=1, day=1, tzinfo=SampleTZInfo())
 
-    @replace('datetime.datetime', test_datetime(None))
-    def test_add_tzinfo_args(self, t):
+    @replace('datetime.datetime', mock_datetime(None))
+    def test_add_tzinfo_args(self, t: Type[MockDateTime]):
         from datetime import datetime
-        with ShouldRaise(TypeError('Cannot add using tzinfo on tdatetime')):
+        datetime = cast(Type[MockDateTime], datetime)
+        with ShouldRaise(TypeError('Cannot add using tzinfo on MockDateTime')):
             datetime.add(2002, 1, 2, 3, 4, 5, 6, SampleTZInfo())
 
     @replace('datetime.datetime',
-             test_datetime(2001, 1, 2, 3, 4, 5, 6, SampleTZInfo()))
+             mock_datetime(2001, 1, 2, 3, 4, 5, 6, SampleTZInfo()))
     def test_max_number_args(self):
         from datetime import datetime
         compare(datetime.now(), d(2001, 1, 2, 3, 4, 5, 6))
 
-    @replace('datetime.datetime', test_datetime(2001, 1, 2))
+    @replace('datetime.datetime', mock_datetime(2001, 1, 2))
     def test_min_number_args(self):
         from datetime import datetime
         compare(datetime.now(), d(2001, 1, 2))
 
-    @replace('datetime.datetime', test_datetime(
+    @replace('datetime.datetime', mock_datetime(
         year=2001,
         month=1,
         day=2,
@@ -289,20 +299,21 @@ class TestDateTime(TestCase):
         from datetime import datetime
         compare(datetime.now(), d(2001, 1, 2, 3, 4, 5, 6))
 
-    @replace('datetime.datetime', test_datetime(2001, 1, 2))
+    @replace('datetime.datetime', mock_datetime(2001, 1, 2))
     def test_utc_now(self):
         from datetime import datetime
         compare(datetime.utcnow(), d(2001, 1, 2))
 
     @replace('datetime.datetime',
-             test_datetime(2001, 1, 2, tzinfo=SampleTZInfo()))
+             mock_datetime(2001, 1, 2, tzinfo=SampleTZInfo()))
     def test_utc_now_with_tz(self):
         from datetime import datetime
         compare(datetime.utcnow(), d(2001, 1, 1, 23, 56))
 
-    @replace('datetime.datetime', test_datetime(strict=True))
+    @replace('datetime.datetime', mock_datetime(strict=True))
     def test_isinstance_strict(self):
         from datetime import datetime
+        datetime = cast(Type[MockDateTime], datetime)
         to_check = []
         to_check.append(datetime(1999, 1, 1))
         to_check.append(datetime.now())
@@ -333,9 +344,32 @@ class TestDateTime(TestCase):
             self.assertTrue(isinstance(inst, d), inst)
             self.assertFalse(inst.__class__ is d, inst)
 
-    @replace('datetime.datetime', test_datetime())
+    def test_strict_addition(self):
+        mock_dt = mock_datetime(strict=True)
+        dt = mock_dt(2001, 1, 1) + timedelta(days=1)
+        assert type(dt) is mock_dt
+
+    def test_non_strict_addition(self):
+        from datetime import datetime
+        mock_dt = mock_datetime(strict=False)
+        dt = mock_dt(2001, 1, 1) + timedelta(days=1)
+        assert type(dt) is datetime
+
+    def test_strict_add(self):
+        mock_dt = mock_datetime(None, strict=True)
+        mock_dt.add(2001, 1, 1)
+        assert type(mock_dt.now()) is mock_dt
+
+    def test_non_strict_add(self):
+        from datetime import datetime
+        mock_dt = mock_datetime(None, strict=False)
+        mock_dt.add(2001, 1, 1)
+        assert type(mock_dt.now()) is datetime
+
+    @replace('datetime.datetime', mock_datetime())
     def test_isinstance_default(self):
         from datetime import datetime
+        datetime = cast(Type[MockDateTime], datetime)
         to_check = []
         to_check.append(datetime(1999, 1, 1))
         to_check.append(datetime.now())
@@ -366,32 +400,36 @@ class TestDateTime(TestCase):
             self.assertTrue(inst.__class__ is d, inst)
 
     def test_subsecond_deltas(self):
-        datetime = test_datetime(delta=0.5)
+        datetime = mock_datetime(delta=0.5)
         compare(datetime.now(), datetime(2001, 1, 1, 0, 0, 0, 0))
         compare(datetime.now(), datetime(2001, 1, 1, 0, 0, 0, 500000))
         compare(datetime.now(), datetime(2001, 1, 1, 0, 0, 1, 0))
 
     def test_ms_delta(self):
-        datetime = test_datetime(delta=100, delta_type='microseconds')
+        datetime = mock_datetime(delta=100, delta_type='microseconds')
         compare(datetime.now(), datetime(2001, 1, 1, 0, 0, 0, 0))
         compare(datetime.now(), datetime(2001, 1, 1, 0, 0, 0, 100))
         compare(datetime.now(), datetime(2001, 1, 1, 0, 0, 0, 200))
 
     def test_tick_when_static(self):
-        datetime = test_datetime(delta=0)
+        datetime = mock_datetime(delta=0)
         compare(datetime.now(), expected=d(2001, 1, 1))
         datetime.tick(hours=1)
         compare(datetime.now(), expected=d(2001, 1, 1, 1))
 
     def test_tick_when_dynamic(self):
         # hopefully not that common?
-        datetime = test_datetime()
+        datetime = mock_datetime()
         compare(datetime.now(), expected=d(2001, 1, 1))
         datetime.tick(hours=1)
         compare(datetime.now(), expected=d(2001, 1, 1, 1, 0, 10))
 
     def test_tick_with_timedelta_instance(self):
-        datetime = test_datetime(delta=0)
+        datetime = mock_datetime(delta=0)
         compare(datetime.now(), expected=d(2001, 1, 1))
         datetime.tick(timedelta(hours=1))
         compare(datetime.now(), expected=d(2001, 1, 1, 1))
+
+    def test_old_import(self):
+        from testfixtures import test_datetime
+        assert test_datetime is mock_datetime
